@@ -17,7 +17,18 @@
 -include_lib("kernel/include/file.hrl").
 
 -compile(export_all).
--export([start_embedded/1, start_embedded/2, start_embedded/3]).  % Thanks!
+-export([start_embedded/1, start_embedded/2, start_embedded/3,   % Thanks!
+	 add_server/2]). 
+-export([new_ssl/0,
+	 ssl_keyfile/1, ssl_keyfile/2,
+	 ssl_certfile/1, ssl_certfile/2,
+	 ssl_verify/1, ssl_verify/2,
+	 ssl_depth/1, ssl_depth/2,
+	 ssl_password/1, ssl_password/2,
+	 ssl_cacertfile/1, ssl_cacertfile/2,
+	 ssl_ciphers/1, ssl_ciphers/2,
+	 ssl_cachetimeout/1, ssl_cachetimeout/2]).
+
 -import(lists, [reverse/1, reverse/2]).
 
 
@@ -44,7 +55,35 @@ start_embedded(DocRoot, SL, GL) when list(DocRoot),list(SL),list(GL) ->
     application:start(yaws),
     GC = setup_gconf(GL, yaws_config:make_default_gconf(false, "")),
     SC = setup_sconf(DocRoot, #sconf{}, SL),
-    yaws_api:setconf(GC, [[SC]]).
+    yaws_config:add_yaws_soap_srv(GC),
+    SCs = yaws_config:add_yaws_auth([SC]),
+    yaws_api:setconf(GC, [SCs]).
+
+add_server(DocRoot, SL) when list(DocRoot),list(SL) ->
+    SC = setup_sconf(DocRoot, #sconf{}, SL),
+    yaws_config:add_sconf(SC).
+
+%%% Access functions for the SSL record.
+new_ssl()             -> #ssl{}.
+%%
+ssl_keyfile(S)      -> S#ssl.keyfile.
+ssl_certfile(S)     -> S#ssl.certfile.
+ssl_verify(S)       -> S#ssl.verify.
+ssl_depth(S)        -> S#ssl.depth.
+ssl_password(S)     -> S#ssl.password.
+ssl_cacertfile(S)   -> S#ssl.cacertfile.
+ssl_ciphers(S)      -> S#ssl.ciphers.
+ssl_cachetimeout(S) -> S#ssl.cachetimeout.
+%%
+ssl_keyfile(S, Keyfile)           -> S#ssl{keyfile = Keyfile}.
+ssl_certfile(S, Certfile)         -> S#ssl{certfile = Certfile}.
+ssl_verify(S, Verify)             -> S#ssl{verify = Verify}.
+ssl_depth(S, Depth)               -> S#ssl{depth = Depth}.
+ssl_password(S, Password)         -> S#ssl{password = Password}.
+ssl_cacertfile(S, Cacertfile)     -> S#ssl{cacertfile = Cacertfile}.
+ssl_ciphers(S, Ciphers)           -> S#ssl{ciphers = Ciphers}.
+ssl_cachetimeout(S, Cachetimeout) -> S#ssl{cachetimeout = Cachetimeout}.
+    
 
 setup_gconf([], GC) -> GC;
 setup_gconf(GL, GC) ->
@@ -53,7 +92,7 @@ setup_gconf(GL, GC) ->
 	   trace = lkup(trace, GL, 
 			GC#gconf.trace),
 	   flags = set_gc_flags(lkup(flags, GL, []), 
-				?GC_DEF),
+				GC#gconf.flags),
 	   logdir = lkup(logdir, GL, 
 			 GC#gconf.logdir),
 	   ebin_dir = lkup(ebin_dir, GL, 
@@ -83,7 +122,9 @@ setup_gconf(GL, GC) ->
 	   id = lkup(id, GL, 
 		     GC#gconf.id),
 	   tmpdir = lkup(tmpdir, GL, 
-			 GC#gconf.tmpdir)
+			 GC#gconf.tmpdir),
+	   enable_soap = lkup(enable_soap, GL, 
+			      GC#gconf.enable_soap)
 	  }.
 
 set_gc_flags([{tty_trace, Bool}|T], Flags) -> 
@@ -775,7 +816,9 @@ to_lower([C|Cs]) when C >= $A, C =< $Z ->
 to_lower([C|Cs]) ->
     [C|to_lower(Cs)];
 to_lower([]) ->
-    [].
+    [];
+to_lower(A) when is_atom(A) ->
+    to_lower(atom_to_list(A)).
 
 
 to_integer(I) when list(I) ->
@@ -1670,8 +1713,7 @@ getuid() ->
  	{win32, _} ->
  	    {ok, "XXX"};
  	_ ->
- 	    erl_ddll:load_driver(filename:dirname(code:which(?MODULE)) ++ 
- 				 "/../priv/", "setuid_drv"),
+	    load_setuid_drv(),
  	    P = open_port({spawn, "setuid_drv g"},[]),
  	    receive
  		{P, {data, "ok " ++ IntList}} ->
@@ -1679,10 +1721,8 @@ getuid() ->
  	    end
     end.
 
-
 idu(User) ->
-    erl_ddll:load_driver(filename:dirname(code:which(?MODULE)) ++ 
- 			 "/../priv/", "setuid_drv"),
+    load_setuid_drv(),
     P = open_port({spawn, "setuid_drv " ++ [$u|User]}, []),
     receive
  	{P, {data, "ok " ++ IntList}} ->
@@ -1690,8 +1730,7 @@ idu(User) ->
     end.
 
 user_to_home(User) ->
-    erl_ddll:load_driver(filename:dirname(code:which(?MODULE)) ++ 
-			 "/../priv/", "setuid_drv"),
+    load_setuid_drv(),
     P = open_port({spawn, "setuid_drv " ++ [$h|User]}, []),
     receive
 	{P, {data, "ok " ++ Home}} ->
@@ -1701,14 +1740,23 @@ user_to_home(User) ->
 
 
 uid_to_name(Uid) ->
-    erl_ddll:load_driver(filename:dirname(code:which(?MODULE)) ++ 
- 			 "/../priv/", "setuid_drv"),
+    load_setuid_drv(),
     P = open_port({spawn, "setuid_drv " ++ [$n|integer_to_list(Uid)]}, []),
     receive
  	{P, {data, "ok " ++ Name}} ->
  	    Name
     end.
 
+load_setuid_drv() ->
+    Path = filename:dirname(code:which(?MODULE)) ++ "/../priv/",
+    case erl_ddll:load_driver(Path, "setuid_drv") of
+	ok ->
+	    ok;
+	{error, Reason} ->
+	    error_logger:format("Failed to load setuid_drv (from ~p) : ~p",
+				[Path, erl_ddll:format_error(Reason)]),
+	    exit(normal)
+    end.
 
 exists(F) ->
     case file:open(F, [read, raw]) of
