@@ -66,8 +66,9 @@ load(E) ->
 	    ?Debug("FLOAD: ~p", [R]),
 	    case R of
 		{ok, GC5, Cs} ->
-		    yaws:mkdir(GC#gconf.tmpdir),
+		    yaws:mkdir(GC5#gconf.tmpdir),
 		    Cs2 = add_yaws_auth(Cs),
+		    add_yaws_soap_srv(GC5),
 		    validate_cs(GC5, Cs2);
 		Err ->
 		    Err
@@ -77,6 +78,19 @@ load(E) ->
     end.
 
 
+add_yaws_soap_srv(GC) when GC#gconf.enable_soap == true ->
+    SoapStarted = (whereis(yaws_soap_srv) /= undefined),
+    if (SoapStarted == false) ->
+	    Spec = {yaws_soap_srv, {yaws_soap_srv, start_link, []},
+		    permanent, 5000, worker, [yaws_soap_srv]},
+	    spawn(fun() -> supervisor:start_child(yaws_sup, Spec) end);
+       true ->
+	    ok
+    end;
+add_yaws_soap_srv(_GC) -> 
+    ok.
+
+
 add_yaws_auth(SCs) ->
     lists:map(
       fun(SC) ->
@@ -297,6 +311,8 @@ yaws_dir() ->
 
 del_tail([_H, ".." |Tail]) ->
     del_tail(Tail);
+del_tail(["..", _H |Tail]) ->
+    del_tail(Tail);
 del_tail([_X, _Y]) ->
     [];
 del_tail([H|T]) ->
@@ -320,6 +336,84 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
     case toks(Chars) of
 	[] ->
 	    fload(FD, globals, GC, C, Cs, Lno+1, Next);
+
+	["subconfig", '=', Name] ->
+	    File = filename:absname(Name),
+	    case is_file(File) of
+		true ->
+		    error_logger:info_msg("Yaws: Using subconfig file ~s~n", [File]),
+		    case file:open(File, [read]) of
+			{ok, FD1} ->
+			    R = (catch fload(FD1, globals, GC, undefined, 
+					     Cs, 1, io:get_line(FD1, ''))),
+			    ?Debug("FLOAD: ~p", [R]),
+			    case R of
+				{ok, GC1, Cs1} ->
+				    fload(FD, globals, GC1, C, Cs1, Lno+1, Next);
+				Err ->
+				    Err
+			    end;
+			_ ->
+			    {error, "Can't open config file" ++ File}
+		    end;
+		false ->
+		    {error, ?F("Expect filename at line ~w", [Lno])}
+	    end;
+
+	["subconfigdir", '=', Name] ->
+	    Dir = filename:absname(Name),
+	    case is_dir(Dir) of
+		true ->
+		    case file:list_dir(Dir) of
+			{ok, Names} ->
+			    Paths = lists:map(
+				      fun(N) ->
+					    filename:absname(N, Dir)
+				      end, Names),
+			    case lists:foldl(
+				   fun(File, Acc) ->
+					case Acc of
+					    {error, _Err} ->
+						Acc;
+					    {ok, GCp, Csp} ->
+						case is_file(File) of
+						    true ->
+							error_logger:info_msg(
+							  "Yaws: Using subconfig file ~s~n",
+							  [File]),
+							case file:open(File, [read]) of
+							    {ok, FD1} ->
+								R = (catch fload(FD1,
+										 globals,
+										 GCp,
+										 undefined, 
+										 Csp,
+										 1,
+										 io:get_line(FD1, ''))),
+								?Debug("FLOAD: ~p", [R]),
+								R;
+							    _ ->
+								{error, "Can't open config file" ++
+									File}
+							end;
+						    false ->
+							%% Ignore subdirectories
+							Acc
+						end
+					end
+				    end, {ok, GC, Cs}, Paths) of
+				{ok, GC1, Cs1} ->
+				    fload(FD, globals, GC1, C, Cs1, Lno+1, Next);
+				Err ->
+				    Err
+			    end;
+			{error, Error} ->
+			    {error, ?F("Direcory ~s is not readable: ~s", [Name, Error])}
+		    end;
+		false ->
+		    {error, ?F("Expect directory at line ~w", [Lno])}
+	    end;
+
 	["trace", '=', Bstr] when GC#gconf.trace == false ->
 	    case Bstr of
 		"traffic" ->
@@ -365,6 +459,15 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 	    fload(FD, globals, GC#gconf{runmods = [Mod|GC#gconf.runmods]},
 		  C, Cs, Lno+1, Next);
 
+	["enable_soap", '=', Bool] ->
+	    if (Bool == "true") ->
+		    fload(FD, globals, GC#gconf{enable_soap = true},
+			  C, Cs, Lno+1, Next);
+	       true ->
+		    fload(FD, globals, GC#gconf{enable_soap = false},
+			  C, Cs, Lno+1, Next)
+	    end;
+
 	["log_wrap_size", '=', Int] ->
 	     case (catch list_to_integer(Int)) of
 		 I when integer(I) ->
@@ -397,17 +500,18 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 	    Dir = filename:absname(Incdir),
 	    case is_dir(Dir) of
 		true ->
-		    fload(FD, globals, GC#gconf{include_dir=[Dir|GC#gconf.include_dir]},
+		    fload(FD, globals, GC#gconf{include_dir=
+						[Dir|GC#gconf.include_dir]},
 			  C, Cs, Lno+1, Next);
 		false ->
 		    {error, ?F("Expect directory at line ~w", [Lno])}
 	    end;
 
-	["tmpdir", '=', Dir] ->
-	     Dir = filename:absname(Dir),
+	["tmpdir", '=', TmpDir] ->
+	    Dir = filename:absname(TmpDir),
 	    case is_dir(Dir) of
 		true ->
-		    fload(FD, globals, GC#gconf{tmpdir=[Dir|GC#gconf.include_dir]},
+		    fload(FD, globals, GC#gconf{tmpdir=Dir},
 			  C, Cs, Lno+1, Next);
 		false ->
 		    {error, ?F("Expect directory at line ~w", [Lno])}
@@ -523,6 +627,15 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 		false ->
 		    {error, ?F("Expect true|false at line ~w", [Lno])}
 	    end;
+	["use_large_ssl_pool", '=',  Bool] ->
+	    case is_bool(Bool) of
+		{true, Val} ->
+		    fload(FD, globals, 
+			  ?gc_set_use_large_ssl_pool(GC,Val),
+			  C, Cs, Lno+1, Next);
+		false ->
+		    {error, ?F("Expect true|false at line ~w", [Lno])}
+	    end;
 	
 
 	['<', "server", Server, '>'] ->  %% first server 
@@ -910,6 +1023,9 @@ fload(FD, server_auth, GC, C, Cs, Lno, Chars, Auth) ->
 	["realm", '=', Realm] ->
 	    A2 = Auth#auth{realm = Realm},
 	    fload(FD, server_auth, GC, C, Cs, Lno+1, Next, A2);
+	["authmod", '=', Mod] ->
+	    A2 = Auth#auth{mod = list_to_atom(Mod)},
+	    fload(FD, server_auth, GC, C, Cs, Lno+1, Next, A2);
 	["user", '=', User] ->
 	    case string:tokens(User, ":") of
 		[Name, Passwd] ->
