@@ -69,20 +69,7 @@ getconf() ->
 stats() -> 
     {S, Time} = status(),
     Diff = calendar:time_difference(Time, calendar:local_time()),
-    G = fun(L) -> reverse(lists:keysort(2, L)) end,
-    R= flatmap(
-	 fun({_Pid, SCS}) ->
-		 map(
-		   fun(SC) ->
-
-			   E = SC#sconf.ets,
-			   L = ets:match(E, {{urlc_total, '$1'}, '$2'}),
-			   {SC#sconf.servername,  
-			    flatten(inet_parse:ntoa(SC#sconf.listen)),
-			    G(map(fun(P) -> list_to_tuple(P) end, L))}
-		   end, SCS)
-	 end, S#state.pairs),
-    {Diff, R}.
+    {Diff, []}.
 
 l2a(L) when list(L) -> list_to_atom(L);
 l2a(A) when atom(A) -> A.
@@ -373,7 +360,7 @@ do_listen(GC, SC) ->
 	    %% Use fdsrv kit from jungerl, requires fdsrv
 	    %% to be properly installed
 	    fdsrv:start(),
-	    case fdsrv:bind_socket(tcp, SC#sconf.port) of
+	    case fdsrv:bind_socket(tcp, {SC#sconf.listen, SC#sconf.port}) of
 		{ok, Fd} ->
 		    {nossl, gen_tcp_listen(SC#sconf.port,[{fd, Fd}|opts(SC)])};
 		Err ->
@@ -404,7 +391,7 @@ gserv(Top, GC, Group0) ->
     SC = hd(Group),
     case do_listen(GC, SC) of
 	{SSLBOOL, {ok, Listen}} ->
-	    call_start_mod(SC),
+	    lists:foreach(fun(XSC) -> call_start_mod(XSC) end, Group),
 	    error_logger:info_msg(
 	      "Yaws: Listening to ~s:~w for servers~s~n",
 	      [inet_parse:ntoa(SC#sconf.listen),
@@ -622,6 +609,8 @@ call_start_mod(SC) ->
 	    Mod = l2a(Mod0),
 	    case code:ensure_loaded(Mod) of
 		{module, Mod} ->
+		    error_logger:info_msg(
+		      "Yaws: calling start_mod: ~p:start/1~n", [Mod]),
 		    spawn(Mod, start, [SC]);
 		Err ->
 		    error_logger:format("Cannot load module ~p: ~p~n", 
@@ -708,7 +697,12 @@ initial_acceptor(GS) ->
 	GS#gs.ssl == nossl ->
 	    acceptor(GS);
 	GS#gs.ssl == ssl ->
-	    initial_acceptor(GS, 5)
+	    GC = GS#gs.gconf,
+	    PoolSize = if
+			   ?gc_use_large_ssl_pool(GC) -> 50;
+			   true -> 8
+		       end,
+	    initial_acceptor(GS, PoolSize)
     end.
 
 
@@ -770,14 +764,24 @@ acceptor0(GS, Top) ->
 		    Top ! {self(), done_client, Int};
 		{'EXIT', normal} ->
 		    exit(normal);
+	        {'EXIT', shutdown} ->
+                    exit(shutdown);
 		{'EXIT', {error, einval}} ->
 		    %% Typically clients that close their end of the socket
 		    %% don't log. Happens all the time.
 		    exit(normal);
+		{'EXIT', {{error, einval}, _}} ->
+		    exit(normal);
+		{'EXIT', {error, closed}} ->
+		    exit(normal);
+		{'EXIT', {{error, closed}, _}} ->
+		    exit(normal);
+		{'EXIT', {{error, econnreset},_}} ->
+		    exit(normal);
 		{'EXIT', Reason} ->
 		    error_logger:error_msg("Yaws process died: ~p~n", 
 					   [Reason]),
-		    exit(normal)
+		    exit(shutdown)
 	    end,
 	    
 	    %% we cache processes
@@ -997,23 +1001,7 @@ maybe_auth_log(Item, ARG) ->
 	    ok;
 	true ->
 	    Req = ARG#arg.req,
-	    CliSock = ARG#arg.clisock,
-	    IP = if
-		     port(CliSock) ->
-			 case inet:peername(CliSock) of
-			     {ok, {Ip, _}} ->
-				 Ip;
-			     _ ->
-				 unknown
-			 end;
-		     true ->
-			 case ssl:peername(CliSock) of
-			     {ok, {Ip, _}} ->
-				 Ip;
-			     _ ->
-				 unknown
-			 end
-		 end,
+	    {IP,_} = ARG#arg.client_ip_port,
 	    Path = safe_decode_path(Req#http_request.path),
 	    yaws_log:authlog(SC#sconf.servername, IP, Path, Item)
     end.
@@ -1292,7 +1280,24 @@ no_body_method(CliSock, Req, Head) ->
 
 make_arg(CliSock, Head, Req, Bin) ->
     SC = get(sc),
+    IP = if
+	     port(CliSock) ->
+		 case inet:peername(CliSock) of
+		     {ok, IpPort} ->
+			 IpPort;
+		     _ ->
+			 {unknown, unknown}
+		 end;
+	     true ->
+		 case ssl:peername(CliSock) of
+		     {ok, IpPort} ->
+			 IpPort;
+		     _ ->
+			 {unknown, unknown}
+		 end
+	 end,
     ARG = #arg{clisock = CliSock,
+	       client_ip_port = IP,
 	       headers = Head,
 	       req = Req,
 	       opaque = SC#sconf.opaque,
@@ -1334,6 +1339,15 @@ handle_request(CliSock, ARG, N) ->
 		    case {IsAuth, 
 			  is_revproxy(DecPath, SC#sconf.revproxy),
 			  is_redirect_map(DecPath, SC#sconf.redirect_map)} of
+			{{appmod, Mod}, _, _} ->
+			    UT = #urltype{type = appmod, 
+			    		  data = {Mod, DecPath},
+			     		  path = DecPath},
+			    ARG2 = ARG#arg{server_path = DecPath,
+					   querydata= QueryPart,
+					   fullpath=UT#urltype.fullpath,
+					   pathinfo=UT#urltype.pathinfo},
+			    handle_ut(CliSock, ARG2, UT, N);
 			{_, _, {true, MethodHostPort}} ->
 			    deliver_302_map(CliSock, Req, ARG, MethodHostPort);
 			{true, false, _} ->
@@ -1346,6 +1360,8 @@ handle_request(CliSock, ARG, N) ->
 			{true, {true, PP}, _} ->
 			    yaws_revproxy:init(CliSock, ARG,
 					       DecPath,QueryPart,PP, N);
+			{false, _, _} ->
+			    deliver_403(CliSock, Req);
 			{{false, Realm}, _, _} ->
 			    deliver_401(CliSock, Req, Realm)
 		    end
@@ -1362,8 +1378,24 @@ handle_request(CliSock, ARG, N) ->
 is_auth(_ARG, _Req_dir, _H, [] ) -> 
     true;
 is_auth(ARG, Req_dir,H,[{Auth_dir, 
-			 #auth{realm=Realm, users=Users, pam=Pam}}|T] ) ->
+			 Auth=#auth{realm=Realm, users=Users, 
+				    pam=Pam, mod=Mod}}|T] ) ->
     case lists:prefix(Auth_dir, Req_dir) of
+	true when Mod /= [] ->
+	    case catch Mod:auth(ARG, Auth) of
+		{'EXIT', _} ->
+		    {false, ""};
+		{appmod, AppMod} ->
+		    {appmod, AppMod};
+		true -> 
+		    true;
+		{false, Realm} ->
+		    maybe_auth_log({401, Realm}, ARG),
+		    {false, Realm};
+		_ ->
+		    maybe_auth_log(403, ARG),
+		    false
+	    end;
 	true ->
 	    case H#headers.authorization of
 		undefined ->
@@ -2844,7 +2876,6 @@ url_type(GetPath, ArgDocroot) ->
     SC=get(sc),
     GC=get(gc),
     E = SC#sconf.ets,
-    update_total(E, GetPath),
     case ets:lookup(E, {url, GetPath}) of
 	[] ->
 	    UT = do_url_type(SC, GetPath, ArgDocroot),
@@ -2905,13 +2936,6 @@ cache_size(_UT) ->
     0.
 
 
-update_total(E, Path) ->
-    case (catch ets:update_counter(E, {urlc_total, Path}, 1)) of
-	{'EXIT', _} ->
-	    ets:insert(E, {{urlc_total, Path}, 1});
-	_ ->
-	    ok
-    end.
 
 
 cache_file(SC, GC, Path, UT) when 
@@ -3015,7 +3039,13 @@ do_url_type(SC, GetPath, ArgDocroot) ->
 		     fullpath = FullPath,
 		     mime = Mime};
 	"/" -> %% special case 
-	    maybe_return_dir(ArgDocroot, GetPath);
+	    case lists:keysearch("/", 1, SC#sconf.appmods) of
+		{value, {_, Mod}} ->
+		    #urltype{type = appmod, 
+			     data = {Mod, []}};
+		_ ->
+		    maybe_return_dir(ArgDocroot, GetPath)
+	    end;
 	[$/, $~ |Tail] ->
 	    ret_user_dir(Tail);
 	_ ->
@@ -3058,14 +3088,18 @@ do_url_type(SC, GetPath, ArgDocroot) ->
 			   [PathElem, Trail, File]),
 		    #urltype{type = appmod, 
 			     data = {Mod, if
-					      Trail==[],PathElem == File ->
+					      Trail==[], PathElem == File ->
 						  [];
 					      hd(File) == $/, 
 					      PathElem == tl(File) ->
 						  [];
 					      true ->
-						  [$/ | conc_path(Trail) ++
-						   File]
+						  case conc_path(Trail) ++ File of
+						      X when hd(X) == $/ ->
+							  X;
+						      X ->
+							  X
+						  end
 					  end},
 			     path = conc_path(Head)}
 	    
@@ -3114,8 +3148,8 @@ check_comps([Pair |Tail], Comps) ->
 				      
 split_at(_, [], _Ack) ->
     false;
-split_at(AM={"/", _Mod}, [_PEslash|Tail], Ack) ->
-    {ok, lists:reverse(Ack), AM, Tail};
+split_at(AM={"/", _Mod}, [PEslash|Tail], Ack) ->
+    {ok, lists:reverse(Ack), AM, [PEslash|Tail]};
 split_at(AM={PE, _Mod}, [PEslash|Tail], Ack) ->
     ?Debug("AM=~p PEslash=~p~n", [AM, PEslash]),
     case no_slash_eq(PE, PEslash) of
